@@ -15,6 +15,10 @@ import sys
 import threading
 
 import numpy
+# numpy's behavoir sometimes changes with versioning, especially in regard as 
+# to when ints are cast to floats.
+# _np_version will be similar to ('1', '13', '1') for which we can use simple comparisons
+_np_version = tuple( ver for ver in numpy.__version__.split('.') )
 
 # Declare a double type that does not exist in Python space
 double = numpy.double
@@ -32,7 +36,7 @@ type_to_kind = {bool: 'bool', int_: 'int', long_: 'long', float: 'float',
                 double: 'double', complex: 'complex', bytes: 'bytes'}
 kind_to_type = {'bool': bool, 'int': int_, 'long': long_, 'float': float,
                 'double': double, 'complex': complex, 'bytes': bytes}
-kind_rank = ['bool', 'int', 'long', 'float', 'double', 'complex', 'none']
+kind_rank = ('bool', 'int', 'long', 'float', 'double', 'complex', 'none')
 scalar_constant_types = [bool, int_, int, float, double, complex, bytes]
 
 # Final corrections for Python 3 (mainly for PyTables needs)
@@ -51,7 +55,10 @@ class Expression(object):
 
     def __getattr__(self, name):
         if name.startswith('_'):
-            return self.__dict__[name]
+            try:
+                return self.__dict__[name]
+            except KeyError:
+                raise AttributeError
         else:
             return VariableNode(name, default_kind)
 
@@ -211,8 +218,7 @@ def func(func, minkind=None, maxkind=None):
 @ophelper
 def where_func(a, b, c):
     if isinstance(a, ConstantNode):
-        #FIXME: This prevents where(True, a, b)
-        raise ValueError("too many dimensions")
+        return b if a.value else c
     if allConstantNodes([a, b, c]):
         return ConstantNode(numpy.where(a, b, c))
     return FuncNode('where', [a, b, c])
@@ -277,8 +283,16 @@ def rtruediv_op(a, b):
 
 @ophelper
 def pow_op(a, b):
+    if (_np_version >= ('1', '12', '0b1') and
+        b.astKind in ('int', 'long') and
+        a.astKind in ('int', 'long') and
+        numpy.any(b.value < 0)):
+
+        raise ValueError(
+            'Integers to negative integer powers are not allowed.')
+
     if allConstantNodes([a, b]):
-        return ConstantNode(a ** b)
+        return ConstantNode(a.value ** b.value)
     if isinstance(b, ConstantNode):
         x = b.value
         if get_optimization() == 'aggressive':
@@ -358,6 +372,8 @@ functions = {
     'expm1': func(numpy.expm1, 'float'),
 
     'abs': func(numpy.absolute, 'float'),
+    'ceil': func(numpy.ceil, 'float', 'double'),
+    'floor': func(numpy.floor, 'float', 'double'),
 
     'where': where_func,
 
