@@ -19,7 +19,7 @@ import numpy as np
 from numpy import (
     array, arange, empty, zeros, int32, int64, uint16, complex_, float64, rec,
     copy, ones_like, where, alltrue, linspace,
-    sum, prod, sqrt, fmod,
+    sum, prod, sqrt, fmod, floor, ceil,
     sin, cos, tan, arcsin, arccos, arctan, arctan2,
     sinh, cosh, tanh, arcsinh, arccosh, arctanh,
     log, log1p, log10, exp, expm1, conj)
@@ -35,10 +35,11 @@ import unittest
 TestCase = unittest.TestCase
 
 double = np.double
-
+if sys.version_info[0] >= 3:
+    long = int
 
 # Recommended minimum versions
-minimum_numpy_version = "1.6"
+minimum_numpy_version = ('1', '7', '0')
 
 
 class test_numexpr(TestCase):
@@ -100,6 +101,12 @@ class test_numexpr(TestCase):
         assert_allclose(evaluate("min(x)"), np.min(x))
         assert_allclose(evaluate("max(x,axis=0)"), np.max(x, axis=0))
 
+        # Fix for #277, array with leading singleton dimension
+        x = np.arange(10).reshape(1,10)
+        assert_allclose(evaluate("sum(x,axis=None)"), sum(x, axis=None) )
+        assert_allclose(evaluate("sum(x,axis=0)"), sum(x, axis=0) )
+        assert_allclose(evaluate("sum(x,axis=1)"), sum(x, axis=1) )
+
         x = arange(10.0)
         assert_allclose(evaluate("sum(x**2+2,axis=0)"), sum(x ** 2 + 2, axis=0))
         assert_allclose(evaluate("prod(x**2+2,axis=0)"), prod(x ** 2 + 2, axis=0))
@@ -284,6 +291,22 @@ class test_numexpr(TestCase):
         res = evaluate('contains(withemptystr, b"")')
         assert_equal(res, [True, True, True])
 
+    def test_str_contains_long_needle(self):
+        a = b'1' + b'a' * 40
+        b = b'a' * 40
+        res = evaluate('contains(a, b)')
+        assert_equal(res, True)
+        
+    def test_where_scalar_bool(self):
+        a = True
+        b = array([1, 2])
+        c = array([3, 4])
+        res = evaluate('where(a, b, c)')
+        assert_array_equal(res, b)
+        a = False
+        res = evaluate('where(a, b, c)')
+        assert_array_equal(res, c)
+
 
 class test_numexpr2(test_numexpr):
     """Testing with 2 threads"""
@@ -510,7 +533,8 @@ func1tests = []
 for func in ['copy', 'ones_like', 'sqrt',
              'sin', 'cos', 'tan', 'arcsin', 'arccos', 'arctan',
              'sinh', 'cosh', 'tanh', 'arcsinh', 'arccosh', 'arctanh',
-             'log', 'log1p', 'log10', 'exp', 'expm1', 'abs', 'conj']:
+             'log', 'log1p', 'log10', 'exp', 'expm1', 'abs', 'conj',
+             'ceil', 'floor']:
     func1tests.append("a + %s(b+c)" % func)
 tests.append(('1_ARG_FUNCS', func1tests))
 
@@ -566,29 +590,50 @@ def test_expressions():
         this_locals = locals()
 
         def method():
-            # We don't want to listen at RuntimeWarnings like
-            # "overflows" or "divide by zero" in plain eval().
-            warnings.simplefilter("ignore")
-            npval = eval(expr, globals(), this_locals)
-            warnings.simplefilter("always")
-            npval = eval(expr, globals(), this_locals)
+            try:
+                # We don't want to listen at RuntimeWarnings like
+                # "overflows" or "divide by zero" in plain eval().
+                warnings.simplefilter("ignore")
+                npval = eval(expr, globals(), this_locals)
+                warnings.simplefilter("always")
+                npval = eval(expr, globals(), this_locals)
+            except Exception as ex:
+                # just store the exception in a variable
+                # compatibility with numpy v1.12
+                # see also https://github.com/pydata/numexpr/issues/239
+                np_exception = ex
+                npval = None
+            else:
+                np_exception = None
+
             try:
                 neval = evaluate(expr, local_dict=this_locals,
                                  optimization=optimization)
+            except AssertionError:
+                raise
+            except NotImplementedError:
+                print('%r not implemented for %s (scalar=%d, opt=%s)'
+                      % (expr, dtype.__name__, test_scalar, optimization))
+            except Exception as ne_exception:
+                same_exc_type = issubclass(type(ne_exception),
+                                           type(np_exception))
+                if np_exception is None or not same_exc_type:
+                    print('numexpr error for expression %r' % (expr,))
+                    raise
+            except:
+                print('numexpr error for expression %r' % (expr,))
+                raise
+            else:
+                msg = ('expected numexpr error not raised for expression '
+                       '%r' % (expr,))
+                assert np_exception is None, msg
+
                 assert equal(npval, neval, exact), """%r
 (test_scalar=%r, dtype=%r, optimization=%r, exact=%r,
  npval=%r (%r - %r)\n neval=%r (%r - %r))""" % (expr, test_scalar, dtype.__name__,
                                                 optimization, exact,
                                                 npval, type(npval), shape(npval),
                                                 neval, type(neval), shape(neval))
-            except AssertionError:
-                raise
-            except NotImplementedError:
-                print('%r not implemented for %s (scalar=%d, opt=%s)'
-                      % (expr, dtype.__name__, test_scalar, optimization))
-            except:
-                print('numexpr error for expression %r' % (expr,))
-                raise
 
         method.description = ('test_expressions(%s, test_scalar=%r, '
                               'dtype=%r, optimization=%r, exact=%r)') \
@@ -626,7 +671,8 @@ def test_expressions():
                     for expr in section_tests:
                         if (dtype == complex and
                             ('<' in expr or '>' in expr or '%' in expr
-                             or "arctan2" in expr or "fmod" in expr)):
+                             or "arctan2" in expr or "fmod" in expr
+                             or "floor" in expr or "ceil" in expr)):
                             # skip complex comparisons or functions not
                             # defined in complex domain.
                             continue
@@ -960,27 +1006,40 @@ class test_subprocess(TestCase):
 
 def print_versions():
     """Print the versions of software that numexpr relies on."""
-    from pkg_resources import parse_version
-    if parse_version(np.__version__) < parse_version(minimum_numpy_version):
-        print("*Warning*: NumPy version is lower than recommended: %s < %s" % \
+    # from pkg_resources import parse_version
+    from numexpr.cpuinfo import cpu
+    import platform
+
+    np_version = tuple( ver for ver in np.__version__.split('.') )
+
+    if minimum_numpy_version < np_version:
+        print('*Warning*: NumPy version is lower than recommended: %s < %s' % \
               (np.__version__, minimum_numpy_version))
     print('-=' * 38)
-    print("Numexpr version:   %s" % numexpr.__version__)
-    print("NumPy version:     %s" % np.__version__)
+    print('Numexpr version:   %s' % numexpr.__version__)
+    print('NumPy version:     %s' % np.__version__)
     print('Python version:    %s' % sys.version)
-    if os.name == 'posix':
-        (sysname, nodename, release, version, machine) = os.uname()
-        print('Platform:          %s-%s' % (sys.platform, machine))
-    print("AMD/Intel CPU?     %s" % numexpr.is_cpu_amd_intel)
-    print("VML available?     %s" % use_vml)
+    (sysname, nodename, release, os_version, machine, processor) = platform.uname()
+    print('Platform:          %s-%s-%s' % (sys.platform, machine, os_version))
+    try:
+        # cpuinfo doesn't work on OSX well it seems, so protect these outputs 
+        # with a try block
+        cpu_info = cpu.info[0]
+        print('CPU vendor:        %s' % cpu_info.get('VendorIdentifier', ''))
+        print('CPU model:         %s' % cpu_info.get('ProcessorNameString', ''))
+        print('CPU clock speed:   %s MHz' % cpu_info.get('~MHz',''))
+    except KeyError:
+        pass
+    print('VML available?     %s' % use_vml)
     if use_vml:
-        print("VML/MKL version:   %s" % numexpr.get_vml_version())
-    print("Number of threads used by default: %d "
-          "(out of %d detected cores)" % (numexpr.nthreads, numexpr.ncores))
+        print('VML/MKL version:   %s' % numexpr.get_vml_version())
+    print('Number of threads used by default: %d '
+          '(out of %d detected cores)' % (numexpr.nthreads, numexpr.ncores))
+    print('Maximum number of threads: %s' % numexpr.MAX_THREADS)
     print('-=' * 38)
 
 
-def test():
+def test(verbosity=1):
     """
     Run all the tests in the test suite.
     """
@@ -990,7 +1049,7 @@ def test():
     # Ignoring them in tests should be ok, as all results are checked out.
     # See https://github.com/pydata/numexpr/issues/183 for details.
     np.seterr(divide='ignore', invalid='ignore', over='ignore', under='ignore')
-    return unittest.TextTestRunner().run(suite())
+    return unittest.TextTestRunner(verbosity=verbosity).run(suite())
 
 
 test.__test__ = False
